@@ -12,9 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import six
-
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.utils.translation import ugettext_lazy as _
 
 from horizon import exceptions
@@ -30,21 +28,41 @@ class ExecuteForm(forms.SelfHandlingForm):
         required=True,
         widget=forms.TextInput(attrs={'readonly': 'readonly'})
     )
-    workflow_input = forms.CharField(
-        label=_("Input"),
-        required=False,
-        initial="{}",
-        widget=forms.widgets.Textarea()
-    )
     task_name = forms.CharField(
         label=_("Task name"),
         required=False,
         widget=forms.TextInput()
     )
 
+    def __init__(self, *args, **kwargs):
+        super(ExecuteForm, self).__init__(*args, **kwargs)
+        self._generate_parameter_fields(kwargs["initial"]["parameter_list"])
+
+    def _generate_parameter_fields(self, list):
+        self.workflow_parameters = []
+        for entry in list.split(","):
+            label, _, default = entry.partition("=")
+            label = label.strip()
+            if label != '':
+                self.workflow_parameters.append(label)
+                if default == "None":
+                    default = None
+                    required = False
+                else:
+                    required = True
+                self.fields[label] = forms.CharField(label=label,
+                                                     required=required,
+                                                     initial=default)
+
     def handle(self, request, data):
         try:
             data['workflow_identifier'] = data.pop('workflow_name')
+            data['workflow_input'] = {}
+            for param in self.workflow_parameters:
+                value = data.pop(param)
+                if value == "":
+                    value = None
+                data['workflow_input'][param] = value
             ex = api.execution_create(request, **data)
 
             msg = _('Execution has been created with id "%s".') % ex.id
@@ -72,6 +90,7 @@ class DefinitionForm(forms.SelfHandlingForm):
         widget=forms.FileInput(
             attrs={'class': 'switched',
                    'data-switch-on': 'definitionsource',
+                   'data-required-when-shown': 'true',
                    'data-definitionsource-file': _('Definition File')}
         ),
         required=False
@@ -82,6 +101,7 @@ class DefinitionForm(forms.SelfHandlingForm):
         widget=forms.widgets.Textarea(
             attrs={'class': 'switched',
                    'data-switch-on': 'definitionsource',
+                   'data-required-when-shown': 'true',
                    'data-definitionsource-raw': _('Definition Data'),
                    'rows': 4}
         ),
@@ -109,7 +129,7 @@ class DefinitionForm(forms.SelfHandlingForm):
                 cleaned_data['definition']
             )
         except Exception as e:
-            raise forms.ValidationError(six.text_type(e))
+            raise forms.ValidationError(str(e))
 
         if not validated.get('valid'):
             raise forms.ValidationError(
@@ -139,8 +159,8 @@ class CreateForm(forms.SelfHandlingForm):
             messages.success(request, msg)
 
             return True
-        except Exception:
-            msg = _('Failed to create workflow.')
+        except Exception as e:
+            msg = _('Failed to create workflow: %s') % e
             redirect = reverse('horizon:mistral:workflows:index')
             exceptions.handle(request, msg, redirect=redirect)
 
@@ -154,7 +174,7 @@ class UpdateForm(CreateForm):
             messages.success(request, msg)
 
             return True
-        except Exception:
-            msg = _('Failed to update workflow.')
+        except Exception as e:
+            msg = _('Failed to update workflow: %s') % e
             redirect = reverse('horizon:mistral:workflows:index')
             exceptions.handle(request, msg, redirect=redirect)
